# -*- coding: utf-8 -*-
'''dLeyna Server Manager mock template'''

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.  See http://www.gnu.org/copyleft/lgpl.html for the full text
# of the license.

__author__ = 'Emanuele Aina'
__email__ = 'emanuele.aina@collabora.com'
__copyright__ = 'Copyright © 2013 Intel Corp. All rights reserved.'
__license__ = 'LGPL 3+'

import copy, dbus, uuid
from dbusmock import MOCK_IFACE, get_object
from items import ITEMS, filter_properties, MEDIA_DEVICE_PROPERTIES, MEDIA_OBJECT2_PROPERTIES, MEDIA_CONTAINER2_PROPERTIES

BUS_NAME = 'com.intel.dleyna-server'
MAIN_OBJ = '/com/intel/dLeynaServer'
MAIN_IFACE = 'com.intel.dLeynaServer.Manager'
SYSTEM_BUS = False

def load(mock, parameters):
    mock.AddMethods(MAIN_IFACE, [
        ('GetServers', '', 'ao', 'ret = self.servers'),
        ('GetVersion', '', 's', 'ret = "0.2.0"'),
        ('PreferLocalAddresses', 'b', '', ''),
        ('Release', '', '', ''),
        ('Rescan', '', '', ''),
        ('SetProtocolInfo', 's', '', ''),
      ])
    mock.AddProperties(MAIN_IFACE, dbus.Dictionary({
        'Foo': 'bar',
      }, signature='sv'))
    mock.next_server_id = 0
    mock.servers = []


@dbus.service.method(MOCK_IFACE,
                     in_signature='', out_signature='o')
def AddServer(self):
    root = '/com/intel/dLeynaServer/server/%d' % (self.next_server_id,)

    # Pre-process the items tree anchoring paths to the new root object
    items = copy.deepcopy(ITEMS)
    for item in items:
        item['Path'] = dbus.ObjectPath(item['Path'].replace('{root}', root))
        item['Parent'] = dbus.ObjectPath(item['Parent'].replace('{root}', root))

    for item in items:
        path = item['Path']
        self.AddObject(path, 'org.gnome.UPnP.MediaObject2', {}, [])
        obj = get_object(path)
        obj.items = items

        if path == root:
            item['FriendlyName'] = 'Mock Server <#{0}>'.format(self.next_server_id)
            item['UDN'] = str(uuid.uuid5(uuid.UUID('9123ef5c-f083-11e2-8000-000000000000'), str(self.next_server_id)))
            obj.AddTemplate("dleynamediadevice.py", filter_properties (item, MEDIA_DEVICE_PROPERTIES))
        obj.AddTemplate("dleynamediaobject.py", filter_properties(item, MEDIA_OBJECT2_PROPERTIES))
        obj.AddTemplate("dleynamediacontainer.py", filter_properties(item, MEDIA_CONTAINER2_PROPERTIES))

    self.servers.append(root)
    self.EmitSignal(MAIN_IFACE, 'FoundServer', 'o', [root])

    self.next_server_id += 1
    return path


@dbus.service.method(MOCK_IFACE,
                     in_signature='o', out_signature='')
def DropServer(self, path):
    self.servers.remove(path)
    self.RemoveObject(path);
    self.EmitSignal(MAIN_IFACE, 'LostServer', 'o', [path])



# -*- coding: utf-8 -*-
'''dLeyna Media Container mock template'''

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.  See http://www.gnu.org/copyleft/lgpl.html for the full text
# of the license.

__author__ = 'Emanuele Aina'
__email__ = 'emanuele.aina@collabora.com'
__copyright__ = 'Copyright © 2013 Intel Corp. All rights reserved.'
__license__ = 'LGPL 3+'

import dbus, os
from dbusmock import MOCK_IFACE, get_object
from items import ITEMS, filter_properties, find_item, find_root, MEDIA_OBJECT2_PROPERTIES, MEDIA_CONTAINER2_PROPERTIES, CHANGE_TYPES
from gi.repository import GLib
from dleynamediadevice import MAIN_IFACE as MEDIA_DEVICE_IFACE

MAIN_IFACE = 'org.gnome.UPnP.MediaContainer2'

def load(mock, parameters):
    mock.AddMethods(MAIN_IFACE, [
        ('CreatePlaylist', 'ssssao', '', ''),
        ('GetCompatibleResource', 'sas', 'a{sv}', ''),
        ('ListChildrenEx', 'uuass', 'aa{sv}', 'ret = []'),
        ('ListContainersEx', 'uuass', 'aa{sv}', 'ret = []'),
        ('ListContainers', 'uuas', 'aa{sv}', 'ret = []'),
        ('ListItemsEx', 'uuass', 'aa{sv}', 'ret = []'),
        ('ListItems', 'uuas', 'aa{sv}', 'ret = []'),
        ('SearchObjectsEx', 'suuass', 'aa{sv}u', 'ret = []'),
        ('SearchObjects', 'suuas', 'aa{sv}u', 'ret = []'),
      ])
    mock.AddProperties(MAIN_IFACE, dbus.Dictionary(parameters, signature='sv'))
    mock.next_upload_id = 0

@dbus.service.method(MAIN_IFACE,
                     in_signature='ssas',
                     out_signature='o')
def CreateContainer(self, display_name, container_type, child_types):
    upload_id = self.next_upload_id
    self.next_upload_id += 1

    path = "{0}/up{1:03}".format(self.__dbus_object_path__, upload_id)
    upload = {
        'ChildCount': dbus.UInt32(0),
        'DisplayName': display_name,
        'Parent': self.__dbus_object_path__,
        'Path': path,
        'Type': 'container',
        'TypeEx': container_type,
      }
    self.items.append(upload)

    self.AddObject(path, 'org.gnome.UPnP.MediaObject2', {}, [])
    obj = get_object(path)
    obj.items = self.items
    obj.AddTemplate("dleynamediaobject.py", filter_properties(upload, MEDIA_OBJECT2_PROPERTIES))
    obj.AddTemplate("dleynamediacontainer.py", filter_properties(upload, MEDIA_CONTAINER2_PROPERTIES))

    device = get_object(find_root(self.items, self.__dbus_object_path__)['Path'])
    device.queue_change ({
        'ChangeType': CHANGE_TYPES['Add'],
        'Path': path
    })

    return (path)

@dbus.service.method(MAIN_IFACE,
                     in_signature='uuas',
                     out_signature='aa{sv}')
def ListChildren(self, offset, count, prop_filter):
    children = [i for i in self.items if i['Parent'] == self.__dbus_object_path__]
    if count:
        children = children[offset:offset+count]
    else:
        children = children[offset:]
    return [filter_properties(i, prop_filter) for i in children]


@dbus.service.method(MAIN_IFACE,
                     in_signature='ss',
                     out_signature='uo')
def Upload(self, display_name, filename):
    upload_id = self.next_upload_id
    self.next_upload_id += 1

    path = "{0}/up{1:03}".format(self.__dbus_object_path__, upload_id)
    size = os.stat(filename).st_size
    upload = {
        'DisplayName': display_name,
        'Parent': self.__dbus_object_path__,
        'Path': path,
        'UploadFilename': filename,
        'UploadId': upload_id,
        'UploadSize': size,
      }
    self.items.append(upload)

    self.AddObject(path, 'org.gnome.UPnP.MediaObject2', {}, [])
    obj = get_object(path)
    obj.items = self.items
    obj.AddTemplate("dleynamediaobject.py", filter_properties(upload, MEDIA_OBJECT2_PROPERTIES))

    device = get_object(find_root(self.items, self.__dbus_object_path__)['Path'])

    def upload_completed():
        device.EmitSignal(MEDIA_DEVICE_IFACE, 'UploadUpdate', 'ustt', (upload_id, 'COMPLETED', size, size))
        device.queue_change ({
            'ChangeType': CHANGE_TYPES['Add'],
            'Path': path
        })
    GLib.idle_add(upload_completed)

    item = find_item(self.items, self.__dbus_object_path__)
    item['ChildCount'] += 1
    return (upload_id, path)



# -*- coding: utf-8 -*-
'''dLeyna DMS fake content hierarchy'''

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.  See http://www.gnu.org/copyleft/lgpl.html for the full text
# of the license.

__author__ = 'Emanuele Aina'
__email__ = 'emanuele.aina@collabora.com'
__copyright__ = 'Copyright © 2013 Intel Corp. All rights reserved.'
__license__ = 'LGPL 3+'

import dbus

def filter_properties(obj, prop_filter):
    return dict(i for i in obj.items() if i[0] in prop_filter)

def find_item(items, path):
    item = [i for i in items if i['Path'] == path]
    if item:
        return item[0]
    return None

def find_root(items, path):
    item = find_item(items, path)
    while item and item['Parent'] != path:
        path = item['Parent']
        item = find_item(items, path)
    return item

MEDIA_DEVICE_PROPERTIES = ['DeviceType', 'UDN', 'FriendlyName', 'IconURL', 'Manufacturer', 'ManufacturerUrl',
                           'ModelDescription', 'ModelName', 'ModelNumber', 'SerialNumber', 'PresentationURL',
                           'DLNACaps', 'SearchCaps', 'SortCaps', 'SortExtCaps', 'FeatureList', 'ServiceResetToken',
                           'SystemUpdateID' ]
MEDIA_OBJECT2_PROPERTIES = [ 'Parent', 'Type', 'TypeEx', 'Path', 'DisplayName' ]
MEDIA_CONTAINER2_PROPERTIES = [ 'ChildCount', 'ItemCount', 'ContainerCount', 'Searchable', 'Resources', 'URLs' ]

CHANGE_TYPES = {
    'Add': dbus.UInt32(1),
    'Mod': dbus.UInt32(2),
    'Del': dbus.UInt32(3),
    'Done': dbus.UInt32(4),
    'Container': dbus.UInt32(5) }

ITEMS = [
    { 'DisplayName': 'The Root',
      'SearchCaps': ['*'],
      'SystemUpdateID': dbus.UInt32(0),
      'Path':   '{root}',
      'Parent': '{root}',
      'Type':   'container',
      'TypeEx': 'container',
      'URLs': [ 'http://127.0.0.1:4242/root/DIDL_S.xml' ] },
    { 'DisplayName': 'Music',
      'Path':   '{root}/1',
      'Parent': '{root}',
      'Type':   'container',
      'TypeEx': 'container' },
    { 'DisplayName': 'A song',
      'Path':   '{root}/11',
      'Parent': '{root}/1',
      'Type':   'music',
      'TypeEx': 'music' },
    { 'DisplayName': 'Another song, with funny chars: < >\'"{1}?#%s❤;',
      'Path':   '{root}/12',
      'Parent': '{root}/1',
      'Type':   'music',
      'TypeEx': 'music' },
    { 'DisplayName': 'Video',
      'Path':   '{root}/2',
      'Parent': '{root}',
      'Type':   'container',
      'TypeEx': 'container' },
    { 'DisplayName': 'This is a movie clip',
      'Path':   '{root}/21',
      'Parent': '{root}/2',
      'Type':   'video',
      'TypeEx': 'video' },
    { 'DisplayName': 'Stuff',
      'Path':   '{root}/3',
      'Parent': '{root}',
      'Type':   'container',
      'TypeEx': 'container',
      'URLs': [ 'http://127.0.0.1:4242/stuff/DIDL_S.xml' ] },
    { 'DisplayName': 'A non-media file',
      'Path':   '{root}/31',
      'Parent': '{root}/3',
      'Type':   'item.unclassified',
      'TypeEx': 'item' },
    { 'DisplayName': 'An empty sub container',
      'Path':   '{root}/32',
      'Parent': '{root}/3',
      'Type':   'container',
      'TypeEx': 'container' },
    { 'DisplayName': 'A picture.jpg',
      'Path':   '{root}/33',
      'Parent': '{root}/3',
      'Type':   'image.photo',
      'TypeEx': 'image.photo',
      'URLs': [ 'http://127.0.0.1:4242/stuff/picture.jpg' ] },
    { 'DisplayName': 'Another picture.jpg',
      'Path':   '{root}/34',
      'Parent': '{root}/3',
      'Type':   'image.photo',
      'TypeEx': 'image.photo',
      'URLs': [ 'http://127.0.0.1:4242/stuff/picture2.jpg' ] },
    { 'DisplayName': 'Another sub container',
      'Path':   '{root}/35',
      'Parent': '{root}/3',
      'Type':   'container',
      'TypeEx': 'container' },
    { 'DisplayName': 'A nested file',
      'Path':   '{root}/351',
      'Parent': '{root}/35',
      'Type':   'item.unclassified',
      'TypeEx': 'item' },
]

# Populate initial ChildCounts
for item in ITEMS:
    if item['Type'].startswith('container'):
        item['ChildCount'] = dbus.UInt32(len([i for i in ITEMS if i != item
            and i['Parent'] == item['Path']]))



# -*- coding: utf-8 -*-
'''dLeyna Media Device mock template'''

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.  See http://www.gnu.org/copyleft/lgpl.html for the full text
# of the license.

__author__ = 'Emanuele Aina'
__email__ = 'emanuele.aina@collabora.com'
__copyright__ = 'Copyright © 2013 Intel Corp. All rights reserved.'
__license__ = 'LGPL 3+'

import dbus, os
from dbusmock import MOCK_IFACE, get_object
from items import find_item, find_root, filter_properties, MEDIA_OBJECT2_PROPERTIES, MEDIA_CONTAINER2_PROPERTIES, CHANGE_TYPES
from gi.repository import GLib

MAIN_IFACE = 'com.intel.dLeynaServer.MediaDevice'

def queue_change(self, change):
    self.changes_id += 1

    if self.changes is None:
        return

    # return only the parent if we want to emulate ContainerUpdateID-generated
    # change notifications
    if not self.changes_detailed:
        parent = find_item(self.items, change['Path'])
        change = filter_properties(parent, 'Path Type TypeEx'.split())
        change['ChangeType'] = CHANGE_TYPES['Container']
        change['UpdateID'] = self.changes_id

    self.changes += (change,)

def load(mock, parameters):
    mock.AddMethods(MAIN_IFACE, [
        ('Cancel', '', '', ''),
        ('CancelUpload', 'u', '', ''),
        ('CreatePlaylistInAnyContainer', 'ssssao', 'uo', ''),
        ('GetUploadIDs', '', 'au', ''),
        ('GetUploadStatus', 'u', 'stt', ''),
      ])
    mock.AddProperties(MAIN_IFACE, dbus.Dictionary(parameters, signature='sv'))
    mock.next_upload_id = 0
    mock.changes = None
    mock.changes_id = 0
    mock.changes_detailed = True
    # make queue_change() a real method of the mock object
    setattr(mock, 'queue_change', queue_change.__get__(mock, mock.__class__))

@dbus.service.method(MAIN_IFACE,
                     in_signature='aoas',
                     out_signature='aa{sv}')
def BrowseObjects(self, object_paths, props):
    results = []
    for object_path in object_paths:
        item = find_item(self.items, object_path)
        if item:
            result = filter_properties(item, props)
        else:
            result = {
                'Error': dbus.Dictionary({
                    'ID': dbus.Int32(42),
                    'Message': 'Object not found'}, 'sv') }
        results.append(result)
    return results

@dbus.service.method(MAIN_IFACE,
                     in_signature='ssas',
                     out_signature='o')
def CreateContainerInAnyContainer(self, display_name, container_type, child_types):
    upload_id = self.next_upload_id
    self.next_upload_id += 1

    path = '{0}/any{1:03}'.format(self.__dbus_object_path__, upload_id)
    upload = {
        'ChildCount': dbus.UInt32(0),
        'DisplayName': display_name,
        'Parent': self.__dbus_object_path__,
        'Path': path,
        'Type': 'container',
        'TypeEx': container_type,
      }
    self.items.append(upload)

    self.AddObject(path, 'org.gnome.UPnP.MediaObject2', {}, [])
    obj = get_object(path)
    obj.items = self.items
    obj.AddTemplate('dleynamediaobject.py', filter_properties(upload, MEDIA_OBJECT2_PROPERTIES))
    obj.AddTemplate('dleynamediacontainer.py', filter_properties(upload, MEDIA_CONTAINER2_PROPERTIES))

    self.queue_change ({
        'ChangeType': CHANGE_TYPES['Add'],
        'Path': path
    })

    return (path)

@dbus.service.method(MOCK_IFACE,
                     in_signature='', out_signature='')
def FlushChanges(self):
    """Emit the currently queued changes"""
    if not self.changes:
        return
    self.EmitSignal(MAIN_IFACE, 'Changed', 'aa{sv}', [self.changes])
    self.changes = []

@dbus.service.method(MOCK_IFACE,
                     in_signature='bb', out_signature='')
def QueueChanges(self, enabled, detailed):
    """Queue a change to be later emitted on FlushChanges()"""
    self.changes_detailed = detailed
    if not enabled:
        self.changes = None
    if enabled and self.changes is None:
        self.changes = []

@dbus.service.method(MAIN_IFACE,
                     in_signature='ss',
                     out_signature='uo')
def UploadToAnyContainer(self, display_name, filename):
    upload_id = self.next_upload_id
    self.next_upload_id += 1

    path = '{0}/any{1:03}'.format(self.__dbus_object_path__, upload_id)
    size = os.stat(filename).st_size
    upload = {
        'DisplayName': display_name,
        'Parent': self.__dbus_object_path__,
        'Path': path,
        'Type': 'item.unclassified',
        'UploadFilename': filename,
        'UploadId': upload_id,
        'UploadSize': size,
      }
    self.items.append(upload)

    self.AddObject(path, 'org.gnome.UPnP.MediaObject2', {}, [])
    obj = get_object(path)
    obj.items = self.items
    obj.AddTemplate('dleynamediaobject.py', filter_properties(upload, MEDIA_OBJECT2_PROPERTIES))

    device = get_object(find_root(self.items, self.__dbus_object_path__)['Path'])

    def upload_completed():
        device.EmitSignal(MAIN_IFACE, 'UploadUpdate', 'ustt', (upload_id, 'COMPLETED', size, size))
        self.queue_change ({
            'ChangeType': CHANGE_TYPES['Add'],
            'Path': path
        })
    GLib.idle_add(upload_completed)

    return (upload_id, path)



# -*- coding: utf-8 -*-
'''dLeyna Media Object mock template'''

# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3 of the License, or (at your option) any
# later version.  See http://www.gnu.org/copyleft/lgpl.html for the full text
# of the license.

__author__ = 'Emanuele Aina'
__email__ = 'emanuele.aina@collabora.com'
__copyright__ = 'Copyright © 2013 Intel Corp. All rights reserved.'
__license__ = 'LGPL 3+'

import dbus
from dbusmock import MOCK_IFACE, get_object
from items import find_item, find_root, CHANGE_TYPES

MAIN_IFACE = 'org.gnome.UPnP.MediaObject2'

def load(mock, parameters):
    mock.AddMethods(MAIN_IFACE, [
        ('GetMetaData', '', 's', ''),
      ])
    mock.AddProperties(MAIN_IFACE, dbus.Dictionary(parameters, signature='sv'))

@dbus.service.method(MAIN_IFACE,
                     in_signature='a{sv}as',
                     out_signature='')
def Update(self, to_add_update, to_delete):
    item = find_item(self.items, self.__dbus_object_path__)
    for prop in to_delete:
        del item[prop]
    for prop, val in to_add_update.items():
        item[prop] = val

    device = get_object(find_root(self.items, self.__dbus_object_path__)['Path'])
    device.queue_change ({
        'ChangeType': CHANGE_TYPES['Mod'],
        'Path': self.__dbus_object_path__
    })

@dbus.service.method(MAIN_IFACE,
                     in_signature='',
                     out_signature='')
def Delete(self):
    device = get_object(find_root(self.items, self.__dbus_object_path__)['Path'])
    device.queue_change ({
        'ChangeType': CHANGE_TYPES['Del'],
        'Path': self.__dbus_object_path__
    })

    item = find_item(self.items, self.__dbus_object_path__)
    self.items.remove(item)

    parent = find_item(self.items, item['Parent'])
    parent['ChildCount'] -= 1
